"use strict";
var querystring = require("querystring");
var request = require("request");
var DEFAULT_HEADERS = {
    'Content-Type': 'multipart/form-data'
};
var Endpoint = (function () {
    function Endpoint(client) {
        this.client = undefined;
        this.baseUrl = undefined;
        this.client = client;
        this.baseUrl = '';
    }
    /**
     * Perform the HTTP request described by the method arguments.
     *
     * @param {string} method   An HTTP verb/method.
     * @param {string} path     The relative path for the resource.
     * @param {object} params   An object with the querystring params.
     */
    Endpoint.prototype.request = function (method, path, params, headers, data) {
        if (params === void 0) { params = {}; }
        if (headers === void 0) { headers = {}; }
        if (data === void 0) { data = {}; }
        // Create the querystring for the given params and add the
        // credentials.
        var options = this.client.options;
        var qs = querystring.stringify(Object.assign({}, params, {
            secret_key: options.credentials.secretKey,
            public_key: options.credentials.publicKey
        }));
        // Discard undefined values in the data object.
        for (var property in data) {
            if (data[property] === undefined) {
                delete data[property];
            }
        }
        // Generate the options object for the http.request method.
        var url = (options.protocol + '://' + options.hostname + options.basePath + path
            + '?' + qs);
        var requestData = {
            method: method.toUpperCase(),
            uri: url,
            headers: Object.assign({}, DEFAULT_HEADERS, headers),
            formData: data
        };
        // Wrap the http.request method in a promise.
        return new Promise(function (resolve, reject) {
            try {
                var req = request(requestData, function (error, res, data) {
                    if (error || res.statusCode >= 400) {
                        return reject(error);
                    }
                    resolve(JSON.parse(data));
                });
            }
            catch (e) {
                reject(e);
            }
        });
    };
    return Endpoint;
}());
exports.Endpoint = Endpoint;
